@@ -1,6 +1,8 @@
 // -*- mode: C++; c-file-style: "cc-mode" -*-
 //*************************************************************************
 //
+// Code available from: https://verilator.org
+//
 // Copyright 2003-2021 by Wilson Snyder. This program is free software; you can
 // redistribute it and/or modify it under the terms of either the GNU
 // Lesser General Public License Version 3 or the Perl Artistic License
@@ -10,20 +12,25 @@
 //*************************************************************************
 ///
 /// \file
-/// \brief Verilator: Common include for OS portability (verilated & verilator)
+/// \brief Verilated/Verilator common header for OS portability
 ///
-///     This header is used by both the Verilator source code (run on the
-///     build and host system), and the Verilated output (run on the target
-///     system).  Code needed by only the host system goes into
-///     config_build.h.in, code needed by Verilated code only goes into
-///     verilated.h, and code needed by both goes here (verilatedos.h).
+/// This header is included by user wrappers and defines the Verilated
+/// public-facing API.
 ///
-/// Code available from: https://verilator.org
+/// User wrapper code does not generally need to include this, instead
+/// include verilated.h.
+///
+/// This header is used by both the Verilator source code (run on the build
+/// and host system), and the Verilated output (run on the target system).
+///
+/// Configuration code needed by only the host system is in
+/// config_build.h.in, code needed by Verilated code only is in
+/// verilated.h, and code needed by both is here (verilatedos.h).
 ///
 //*************************************************************************
 
 #ifndef VERILATOR_VERILATEDOS_H_
-#define VERILATOR_VERILATEDOS_H_  ///< Header Guard
+#define VERILATOR_VERILATEDOS_H_
 
 // Current clang-format versions botch #ifdef inclusion, so
 // clang-format off
@@ -36,13 +43,18 @@
 # define VL_ATTR_COLD __attribute__((cold))
 # define VL_ATTR_HOT __attribute__((hot))
 # define VL_ATTR_NORETURN __attribute__((noreturn))
+// clang and gcc-8.0+ support no_sanitize("string") style attribute
+# if defined(__clang__) || (__GNUC__ >= 8)
+#  define VL_ATTR_NO_SANITIZE_ALIGN __attribute__((no_sanitize("alignment")))
+#else  // The entire undefined sanitizer has to be disabled for older gcc
+#  define VL_ATTR_NO_SANITIZE_ALIGN __attribute__((no_sanitize_undefined))
+#endif
 # define VL_ATTR_PRINTF(fmtArgNum) __attribute__((format(printf, (fmtArgNum), (fmtArgNum) + 1)))
 # define VL_ATTR_PURE __attribute__((pure))
 # define VL_ATTR_UNUSED __attribute__((unused))
 # if !defined(_WIN32) && !defined(__MINGW32__)
 #  define VL_ATTR_WEAK __attribute__((weak))
 # endif
-# define VL_FUNC __func__
 # if defined(__clang__) && defined(VL_THREADED)
 #  define VL_ACQUIRE(...) __attribute__((acquire_capability(__VA_ARGS__)))
 #  define VL_ACQUIRE_SHARED(...) __attribute__((acquire_shared_capability(__VA_ARGS__)))
@@ -61,40 +73,38 @@
 # define VL_UNREACHABLE __builtin_unreachable();
 # define VL_PREFETCH_RD(p) __builtin_prefetch((p), 0)
 # define VL_PREFETCH_RW(p) __builtin_prefetch((p), 1)
-#elif defined(_MSC_VER)
-# define VL_FUNC __FUNCTION__
 #endif
 
 // Defaults for unsupported compiler features
 #ifndef VL_ATTR_ALIGNED
-# define VL_ATTR_ALIGNED(alignment)  ///< Align structure to specified byte alignment
+# define VL_ATTR_ALIGNED(alignment)  ///< Attribute to align structure to byte alignment
 #endif
 #ifndef VL_ATTR_ALWINLINE
-# define VL_ATTR_ALWINLINE  ///< Inline, even when not optimizing
+# define VL_ATTR_ALWINLINE  ///< Attribute to inline, even when not optimizing
 #endif
 #ifndef VL_ATTR_COLD
-# define VL_ATTR_COLD  ///< Function is rarely executed
+# define VL_ATTR_COLD  ///< Attribute that function is rarely executed
 #endif
 #ifndef VL_ATTR_HOT
-# define VL_ATTR_HOT  ///< Function is highly executed
+# define VL_ATTR_HOT  ///< Attribute that function is highly executed
 #endif
 #ifndef VL_ATTR_NORETURN
-# define VL_ATTR_NORETURN  ///< Function does not ever return
+# define VL_ATTR_NORETURN  ///< Attribute that function does not ever return
+#endif
+#ifndef VL_ATTR_NO_SANITIZE_ALIGN
+# define VL_ATTR_NO_SANITIZE_ALIGN ///< Attribute that the function contains intended unaligned access
 #endif
 #ifndef VL_ATTR_PRINTF
-# define VL_ATTR_PRINTF(fmtArgNum)  ///< Function with printf format checking
+# define VL_ATTR_PRINTF(fmtArgNum)  ///< Attribute for function with printf format checking
 #endif
 #ifndef VL_ATTR_PURE
-# define VL_ATTR_PURE  ///< Function is pure (and thus also VL_MT_SAFE)
+# define VL_ATTR_PURE  ///< Attribute that function is pure (and thus also VL_MT_SAFE)
 #endif
 #ifndef VL_ATTR_UNUSED
-# define VL_ATTR_UNUSED  ///< Function that may be never used
+# define VL_ATTR_UNUSED  ///< Attribute that function that may be never used
 #endif
 #ifndef VL_ATTR_WEAK
-# define VL_ATTR_WEAK  ///< Function external that is optionally defined
-#endif
-#ifndef VL_FUNC
-# define VL_FUNC "__func__"  ///< Name of current function for error macros
+# define VL_ATTR_WEAK  ///< Attribute that function external that is optionally defined
 #endif
 #ifndef VL_CAPABILITY
 # define VL_ACQUIRE(...)  ///< Function requires a capability/lock (-fthread-safety)
@@ -110,18 +120,19 @@
 # define VL_SCOPED_CAPABILITY  ///< Scoped threaded capability/lock (-fthread-safety)
 #endif
 #ifndef VL_LIKELY
-# define VL_LIKELY(x) (!!(x))  ///< Boolean expression more often true than false
-# define VL_UNLIKELY(x) (!!(x))  ///< Boolean expression more often false than true
+# define VL_LIKELY(x) (!!(x))  ///< Return boolean expression that is more often true
+# define VL_UNLIKELY(x) (!!(x))  ///< Return boolean expression that is more often false
 #endif
-# define VL_UNCOVERABLE(x) VL_UNLIKELY(x)  ///< Boolean expression never hit by users (no coverage)
+/// Boolean expression never hit by users (branch coverage disabled)
+# define VL_UNCOVERABLE(x) VL_UNLIKELY(x)
 #ifndef VL_UNREACHABLE
-# define VL_UNREACHABLE  ///< Point that may never be reached
+# define VL_UNREACHABLE  ///< Statement that may never be reached (for coverage etc)
 #endif
 #ifndef VL_PREFETCH_RD
-# define VL_PREFETCH_RD(p)  ///< Prefetch data with read intent
+# define VL_PREFETCH_RD(p)  ///< Prefetch pointer argument with read intent
 #endif
 #ifndef VL_PREFETCH_RW
-# define VL_PREFETCH_RW(p)  ///< Prefetch data with read/write intent
+# define VL_PREFETCH_RW(p)  ///< Prefetch pointer argument with read/write intent
 #endif
 
 #if defined(VL_THREADED) && !defined(VL_CPPCHECK)
@@ -134,31 +145,39 @@
 # else
 #  error "Unsupported compiler for VL_THREADED: No thread-local declarator"
 # endif
-# define VL_THREAD_LOCAL thread_local  ///< Use new C++ static local thread
+# define VL_THREAD_LOCAL thread_local  // "thread_local" when supported
 #else
-# define VL_THREAD_LOCAL  ///< Use new C++ static local thread
+# define VL_THREAD_LOCAL  // "thread_local" when supported
 #endif
 
 #ifndef VL_NO_LEGACY
-# define VL_THREAD  ///< Deprecated
-# define VL_STATIC_OR_THREAD static  ///< Deprecated
+# define VL_FUNC __func__  // Deprecated
+# define VL_THREAD  // Deprecated
+# define VL_STATIC_OR_THREAD static  // Deprecated
 #endif
 
-#define VL_PURE  ///< Comment tag that Function is pure (and thus also VL_MT_SAFE)
-#define VL_MT_SAFE  ///< Comment tag that function is threadsafe when VL_THREADED
-#define VL_MT_SAFE_POSTINIT  ///< Comment tag that function is threadsafe when VL_THREADED, only
-                             ///< during normal operation (post-init)
-#define VL_MT_SAFE_EXCLUDES(mutex) VL_EXCLUDES(mutex)  ///< Threadsafe and uses given mutex
-#define VL_MT_UNSAFE  ///< Comment tag that function is not threadsafe when VL_THREADED
-#define VL_MT_UNSAFE_ONE  ///< Comment tag that function is not threadsafe when VL_THREADED,
-                          ///< protected to make sure single-caller
+// Comment tag that Function is pure (and thus also VL_MT_SAFE)
+#define VL_PURE
+// Comment tag that function is threadsafe when VL_THREADED
+#define VL_MT_SAFE
+// Comment tag that function is threadsafe when VL_THREADED, only
+// during normal operation (post-init)
+#define VL_MT_SAFE_POSTINIT
+// Attribute that function is clang threadsafe and uses given mutex
+#define VL_MT_SAFE_EXCLUDES(mutex) VL_EXCLUDES(mutex)
+// Comment tag that function is not threadsafe when VL_THREADED
+#define VL_MT_UNSAFE
+// Comment tag that function is not threadsafe when VL_THREADED,
+// protected to make sure single-caller
+#define VL_MT_UNSAFE_ONE
 
 #ifndef VL_NO_LEGACY
-# define VL_ULL(c) (c##ULL)  ///< Add appropriate suffix to 64-bit constant (deprecated)
+# define VL_ULL(c) (c##ULL)  // Add appropriate suffix to 64-bit constant (deprecated)
 #endif
 
-// This is not necessarily the same as #UL, depending on what the IData typedef is.
-#define VL_UL(c) (static_cast<IData>(c##UL))  ///< Add appropriate suffix to 32-bit constant
+// Convert argument to IData
+// This is not necessarily the same as "#UL", depending on what the IData typedef is.
+#define VL_UL(c) (static_cast<IData>(c##UL))
 
 #if defined(VL_CPPCHECK) || defined(__clang_analyzer__) || __cplusplus < 201103L
 # define VL_DANGLING(var)
@@ -225,7 +244,7 @@
 extern "C" {
 void __gcov_flush();  // gcc sources gcc/gcov-io.h has the prototype
 }
-/// Flush internal code coverage data before e.g. abort()
+// Flush internal code coverage data before e.g. std::abort()
 # define VL_GCOV_FLUSH() \
     __gcov_flush()
 #else
@@ -377,23 +396,23 @@ typedef unsigned long long vluint64_t;  ///< 64-bit unsigned type
 #define VL_CACHE_LINE_BYTES 64  ///< Bytes in a cache line (for alignment)
 
 #ifndef VL_NO_LEGACY
-# define VL_WORDSIZE VL_IDATASIZE  ///< Legacy define
+# define VL_WORDSIZE VL_IDATASIZE  // Legacy define
 #endif
 
-/// Bytes this number of bits needs (1 bit=1 byte)
+/// Return number of bytes argument-number of bits needs (1 bit=1 byte)
 #define VL_BYTES_I(nbits) (((nbits) + (VL_BYTESIZE - 1)) / VL_BYTESIZE)
-/// Words/EDatas this number of bits needs (1 bit=1 word)
+/// Return Words/EDatas in argument-number of bits needs (1 bit=1 word)
 #define VL_WORDS_I(nbits) (((nbits) + (VL_EDATASIZE - 1)) / VL_EDATASIZE)
-/// Words/EDatas a quad requires
+// Number of Words/EDatas a quad requires
 #define VL_WQ_WORDS_E VL_WORDS_I(VL_QUADSIZE)
 
 //=========================================================================
 // Class definition helpers
 
-/// Used to indicate a base class, e.g. cannot label "class final"
+// Comment tag to indicate a base class, e.g. cannot label "class final"
 #define VL_NOT_FINAL
 
-/// Used to declare a class as uncopyable; put after a private:
+// Declare a class as uncopyable; put after a private:
 #define VL_UNCOPYABLE(Type) \
     Type(const Type& other) = delete; \
     Type& operator=(const Type&) = delete
@@ -402,7 +421,8 @@ typedef unsigned long long vluint64_t;  ///< 64-bit unsigned type
 // Verilated function size macros
 
 #define VL_MULS_MAX_WORDS 16  ///< Max size in words of MULS operation
-#define VL_TO_STRING_MAX_WORDS 64  ///< Max size in words of String conversion operation
+#define VL_VALUE_STRING_MAX_WORDS 64  ///< Max size in words of String conversion operation
+#define VL_VALUE_STRING_MAX_CHARS (VL_VALUE_STRING_MAX_WORDS * VL_EDATASIZE / VL_BYTESIZE)
 
 //=========================================================================
 // Base macros
@@ -411,14 +431,15 @@ typedef unsigned long long vluint64_t;  ///< 64-bit unsigned type
 #define VL_SIZEBITS_Q (VL_QUADSIZE - 1)  ///< Bit mask for bits in a quad
 #define VL_SIZEBITS_E (VL_EDATASIZE - 1)  ///< Bit mask for bits in a quad
 
-/// Mask for words with 1's where relevant bits are (0=all bits)
+/// Return mask for words with 1's where relevant bits are (0=all bits)
 #define VL_MASK_I(nbits) (((nbits) & VL_SIZEBITS_I) ? ((1U << ((nbits) & VL_SIZEBITS_I)) - 1) : ~0)
-/// Mask for quads with 1's where relevant bits are (0=all bits)
+/// Return mask for quads with 1's where relevant bits are (0=all bits)
 #define VL_MASK_Q(nbits) \
     (((nbits) & VL_SIZEBITS_Q) ? ((1ULL << ((nbits) & VL_SIZEBITS_Q)) - 1ULL) : ~0ULL)
-/// Mask for EData with 1's where relevant bits are (0=all bits)
+/// Return mask for EData with 1's where relevant bits are (0=all bits)
 #define VL_MASK_E(nbits) VL_MASK_I(nbits)
-#define VL_EUL(n) VL_UL(n)  ///< Make constant number EData sized
+
+#define VL_EUL(n) VL_UL(n)  // Make constant number EData sized
 
 #define VL_BITWORD_I(bit) ((bit) / VL_IDATASIZE)  ///< Word number for sv DPI vectors
 #define VL_BITWORD_E(bit) ((bit) >> VL_EDATASIZE_LOG2)  ///< Word number for a wide quantity
@@ -431,19 +452,19 @@ typedef unsigned long long vluint64_t;  ///< 64-bit unsigned type
 // #defines, to avoid requiring math.h on all compile runs
 
 #ifdef _MSC_VER
-# define VL_TRUNC(n) (((n) < 0) ? ceil((n)) : floor((n)))
-# define VL_ROUND(n) (((n) < 0) ? ceil((n)-0.5) : floor((n) + 0.5))
+# define VL_TRUNC(n) (((n) < 0) ? std::ceil((n)) : std::floor((n)))
+# define VL_ROUND(n) (((n) < 0) ? std::ceil((n)-0.5) : std::floor((n) + 0.5))
 #else
-# define VL_TRUNC(n) trunc(n)
-# define VL_ROUND(n) round(n)
+# define VL_TRUNC(n) std::trunc(n)
+# define VL_ROUND(n) std::round(n)
 #endif
 
 //=========================================================================
 // Performance counters
 
-/// The vluint64_t argument is loaded with a high-performance counter for profiling
-/// or 0x0 if not implemented on this platform
 #if defined(__i386__) || defined(__x86_64__)
+// The vluint64_t argument is loaded with a high-performance counter for profiling
+// or 0x0 if not implemented on this platform
 #define VL_RDTSC(val) \
     { \
         vluint32_t hi, lo; \
@@ -467,8 +488,8 @@ typedef unsigned long long vluint64_t;  ///< 64-bit unsigned type
 #  include "Windows.h"
 #  define VL_CPU_RELAX() YieldProcessor()
 # elif defined(__i386__) || defined(__x86_64__) || defined(VL_CPPCHECK)
-/// For more efficient busy waiting on SMT CPUs, let the processor know
-/// we're just waiting so it can let another thread run
+// For more efficient busy waiting on SMT CPUs, let the processor know
+// we're just waiting so it can let another thread run
 #  define VL_CPU_RELAX() asm volatile("rep; nop" ::: "memory")
 # elif defined(__ia64__)
 #  define VL_CPU_RELAX() asm volatile("hint @pause" ::: "memory")
@@ -490,7 +511,9 @@ typedef unsigned long long vluint64_t;  ///< 64-bit unsigned type
 # define VL_STRCASECMP strcasecmp
 #endif
 
-#ifdef _MSC_VER
+#ifdef __MINGW32__
+# define VL_LOCALTIME_R(timep, tmp) localtime_s((tmp), (timep))
+#elif defined(_MSC_VER)
 # define VL_LOCALTIME_R(timep, tmp) localtime_c((tmp), (timep))
 #else
 # define VL_LOCALTIME_R(timep, tmp) localtime_r((timep), (tmp))
